import os

from dbt.compilation import Compiler
from dbt.config import RuntimeConfig
import dbt.adapters.factory

import dbt.loader
import dbt.ui
from dbt.logger import GLOBAL_LOGGER as logger
from dbt.task.generate import unflatten
from jinja2 import Template


SCHEMA_YML_TEMPLATE = Template(
    """
version: 2
models:
{% for model_dict in models %}
  - name: {{model_dict['name']}}
    description: 'TODO: Replace me'
    columns:
    {% for col in model_dict['columns'] -%}
    - name: {{col['name']}}
    {% endfor %}
    {% endfor %}
"""
)


class BootstrapTask:
    def __init__(self, args):
        self.args = args
        self.config = RuntimeConfig.from_args(args)

    def _get_manifest(self):
        compiler = Compiler(self.config)
        compiler.initialize()

        all_projects = compiler.get_all_projects()

        manifest = dbt.loader.GraphLoader.load_all(self.config, all_projects)
        return manifest

    def render_relations(self, models):
        return SCHEMA_YML_TEMPLATE.render(models=models)

    def write_relation(self, design_file_path, yml):
        if os.path.isfile(design_file_path):
            logger.info(
                dbt.ui.printer.yellow(
                    "Warning: File {} already exists. Skipping".format(design_file_path)
                )
            )
            return

        logger.info("Creating design file: {}".format(design_file_path))
        with open(design_file_path, "w") as f:
            f.write(yml)

    def print_relation(self, yml):
        logger.info(yml)

    def prep_metadata(self, meta_dict):
        columns = []
        for colname in meta_dict["columns"]:
            column = {}
            column["name"] = colname
            columns.append(column)

        model = {}
        model["name"] = meta_dict["metadata"]["name"]
        if meta_dict["metadata"]["comment"]:
            description = meta_dict["metadata"]["comment"]
        else:
            description = "TODO: Replace me"

        model["description"] = description
        model["columns"] = columns

        return model

    def run(self):
        single_file = self.args.single_file
        write_files = self.args.write_files
        schemas = self.args.schemas

        logger.info("Bootstrapping the following schemas:")
        for schema in schemas:
            logger.info("- {}".format(schema))

        # Look up all of the relations in the DB
        manifest = self._get_manifest()
        adapter = dbt.adapters.factory.get_adapter(self.config)
        all_relations = adapter.get_catalog(manifest)

        selected_relations = all_relations.where(
            lambda row: row["table_schema"] in schemas
        )

        zipped_relations = [
            dict(zip(selected_relations.column_names, row))
            for row in selected_relations
        ]

        relations_to_design = unflatten(zipped_relations)

        if len(relations_to_design) == 0:
            logger.info(
                dbt.ui.printer.yellow(
                    "Warning: No relations found in selected schemas: {}."
                    "\nAborting.".format(schemas)
                )
            )
            return {}

        for schema, relations in relations_to_design.items():
            schema_path = os.path.join(self.config.source_paths[0], schema)
            if not write_files:
                pass
            elif os.path.isdir(schema_path):
                pass
            else:
                os.mkdir(schema_path)

            all_models = []

            for relation, meta_data in relations.items():

                relation_dict = self.prep_metadata(meta_data)
                all_models.append(relation_dict)

                if not single_file:
                    if not write_files:
                        logger.info("-" * 20)
                        logger.info(
                            "Design for relation: {}.{}".format(schema, relation)
                        )
                        logger.info("-" * 20)
                        yml = self.render_relations([relation_dict])
                        self.print_relation(yml)
                    else:
                        design_file_name = "{}.yml".format(relation)
                        design_file_path = os.path.join(schema_path, design_file_name)
                        yml = self.render_relations([relation_dict])
                        self.write_relation(design_file_path, yml)

            if single_file:
                if not write_files:
                    logger.info("-" * 20)
                    logger.info("Design for schmea: {}".format(schema))
                    logger.info("-" * 20)
                    yml = self.render_relations(all_models)
                    self.print_relation(yml)

                else:
                    design_file_name = "{}.yml".format(schema)
                    design_file_path = os.path.join(schema_path, design_file_name)
                    yml = self.render_relations([all_models])
                    self.write_relation(design_file_path, yml)

        return all_models

    def interpret_results(self, results):
        return len(results) != 0



import argparse
import sys
import os

import core.bootstrap as bootstrap_task
import core.compare as compare_task

from dbt.config import PROFILES_DIR


def get_nearest_project_dir():
    root_path = os.path.abspath(os.sep)
    cwd = os.getcwd()

    while cwd != root_path:
        project_file = os.path.join(cwd, "dbt_project.yml")
        if os.path.exists(project_file):
            return cwd
        cwd = os.path.dirname(cwd)

    return None


def parse_args(args):

    p = argparse.ArgumentParser(
        prog="dbt-helper: additional tools for wokring with DBT",
        formatter_class=argparse.RawTextHelpFormatter,
        description="Additional CLI tools for faster DBT development and database management",
        epilog="Select one of these sub-commands and you can find more help from there.",
    )

    base_subparser = argparse.ArgumentParser(add_help=False)
    base_subparser.add_argument(
        "--profiles-dir",
        default=PROFILES_DIR,
        type=str,
        help="""
        Which directory to look in for the profiles.yml file. Default = {}
        """.format(
            PROFILES_DIR
        ),
    )
    base_subparser.add_argument(
        "--profile",
        required=False,
        type=str,
        help="""
        Which profile to load. Overrides setting in dbt_project.yml.
        """,
    )
    base_subparser.add_argument(
        "--target",
        default=None,
        type=str,
        help="Which target to load for the given profile",
    )

    subs = p.add_subparsers(title="Available sub-commands", dest="command")

    compare_sub = subs.add_parser(
        "compare",
        parents=[base_subparser],
        help="Compare your dbt project specifications with what's in your database.",
    )
    compare_sub.set_defaults(cls=compare_task.CompareTask, which="compare")

    bootstrap_sub = subs.add_parser(
        "bootstrap",
        parents=[base_subparser],
        help="Bootstrap schema.yml files from database catalog",
    )
    bootstrap_sub.set_defaults(cls=bootstrap_task.BootstrapTask, which="bootsrap")

    bootstrap_sub.add_argument(
        "--schemas",
        required=True,
        nargs="+",
        help="""
        Required. Specify the schemas to inspect when bootstrapping
        schema.yml files.
        """,
    )
    bootstrap_sub.add_argument(
        "--single-file",
        action="store_true",
        dest="single_file",
        help="Store all of the schema information in a single schema.yml file",
    )
    bootstrap_sub.add_argument(
        "--write-files",
        action="store_true",
        dest="write_files",
        help="Create schema.yml files (will not over-write existing files).",
    )

    if len(args) == 0:
        p.print_help()
        sys.exit(1)
        return

    parsed = p.parse_args(args)
    return parsed


def handle(args):

    nearest_project_dir = get_nearest_project_dir()
    if nearest_project_dir is None:
        raise Exception(
            "fatal: Not a dbt project (or any of the parent directories). "
            "Missing dbt_project.yml file"
        )

    os.chdir(nearest_project_dir)

    parsed = parse_args(args)
    results = None

    if parsed.command == "bootstrap":
        task = bootstrap_task.BootstrapTask(parsed)
        results = task.run()

    if parsed.command == "compare":
        task = compare_task.CompareTask(parsed)
        results = task.run()

    return results


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    handle(args)


if __name__ == "__main__":
    main()



from setuptools import find_packages, setup


package_name = "dbt-helper"
package_version = "0.0.2"
description = """dbt-helper is a command line tool to help ease dbt development and database management"""


setup(
    name=package_name,
    version=package_version,
    description=description,
    author="Michael Kaminsky",
    author_email="michael@kaminsky.rocks",
    url="https://github.com/mikekaminsky/dbt-helper",
    packages=find_packages(),
    package_data={},
    test_suite="test",
    entry_points={"console_scripts": ["dbt-helper = core.main:main"]},
    scripts=[],
    install_requires=["dbt"],
)
